package com.google.gwt.misge.alarms.server;

import java.util.ArrayList;
import java.util.Collections;

import javax.management.AttributeNotFoundException;
import javax.management.InstanceNotFoundException;
import javax.management.MBeanException;
import javax.management.MBeanServer;
import javax.management.Notification;
import javax.management.ObjectName;
import javax.management.ReflectionException;

import org.jboss.monitor.alarm.AlarmHelper;
import org.jboss.monitor.alarm.AlarmNotification;
import org.jboss.monitor.alarm.AlarmTableNotification;
import org.jboss.monitor.services.ActiveAlarmTableMBean;
import org.jboss.mx.util.MBeanServerLocator;
import org.jboss.util.Strings;

import com.google.gwt.misge.alarms.client.AlarmComparator;
import com.google.gwt.misge.alarms.client.GWTAlarm;
import com.google.gwt.user.server.rpc.RemoteServiceServlet;

/*
 * @author jiorgos |  misge.intra@gmail.com under LGPL Licence and WITHOUT ANY WARRANTY
 */
public class JBossAlarmServiceImpl extends RemoteServiceServlet implements
        com.google.gwt.misge.alarms.client.AlarmService {

    /**
     * 
     */
    private static final long serialVersionUID = 3257852099311383347L;
    private final static String STATIC_STRING = "java.lang.String";

    private MBeanServer server = null;
    private ObjectName target = null;
    private static AlarmTableNotification[] altab = null;
    private static GWTAlarm alarmSize = null;
    
    
    public JBossAlarmServiceImpl() {
        super();
        server = MBeanServerLocator.locateJBoss();
        target = ActiveAlarmTableMBean.OBJECT_NAME;
    }

    
    public GWTAlarm getLastAlarm (){
        if(alarmSize == null)
            return new GWTAlarm(0);
        else
            return alarmSize;
    }
    
    /* (non-Javadoc)
     * @see com.google.gwt.misge.alarms.client.AlarmService#getAlarms()
     */
    public GWTAlarm[] getAlarms() {
        try {
            altab = (AlarmTableNotification[]) server.invoke(target,
                    "fetchAlarmTable", new Object[] {}, new String[] {});

        } catch (Exception e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
        return getAlarms("ALL");
    }

    public GWTAlarm[] getAlarms(String filter) {
        
        
        try {
            // if( altab == null){
            altab = (AlarmTableNotification[]) server.invoke(target,
                    "fetchAlarmTable", new Object[] {}, new String[] {});
            // }

        } catch (InstanceNotFoundException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        } catch (MBeanException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        } catch (ReflectionException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
        ArrayList values = new ArrayList();

        for (int i = 0; i < altab.length; i++) {
            AlarmTableNotification atn = altab[i];

            String severity = AlarmHelper.getSeverityAsString(atn.getSeverity());

            String alarmId = atn.getAlarmId();
            String alarmState = AlarmHelper.getStateAsString(atn
                    .getAlarmState());
//            boolean ackState = atn.getAckState();
//            long ackTime = atn.getAckTime();
//            String ackUser = atn.getAckUser();
//            String ackSystem = atn.getAckSystem();

            Notification n = (Notification) atn.getUserData();
            Object source = AlarmNotification.getEffectiveSource(n);
            // String sourceLink =
            // "/jmx-console/HtmlAdaptor?action=inspectMBean&name=" +
            // URLEncoder.encode(source.toString(),"iso-8859");
            String type = n.getType();
            // reply += n.getMessage() + "\n";

            long timeStamp = n.getTimeStamp();
            long sequenceNumber = n.getSequenceNumber();
            String message = (n.getMessage() != null) ? Strings.subst("\n",
                    "<br>", n.getMessage()) : "null";
            String userData = (n.getUserData() != null) ? Strings.subst("\n",
                    "<br>", n.getUserData().toString()) : "null";
            // System.out.println("severity: "+severity + " filter:"+filter);
            if (severity.equals(filter)) {
                values.add(new GWTAlarm(alarmId, type, source.toString(),
                        userData, severity, message, alarmState, timeStamp,sequenceNumber));
            } else if (filter.equalsIgnoreCase("ALL") || filter == null) {
                values.add(new GWTAlarm(alarmId, type, source.toString(),
                        userData, severity, message, alarmState, timeStamp,
                        sequenceNumber));
            }

        }
        if(altab.length > 0){
            Collections.sort(values, new AlarmComparator());
            alarmSize = new GWTAlarm(altab.length);
        
            return (GWTAlarm[]) values.toArray(new GWTAlarm[values.size()]);
        }else{
            alarmSize = new GWTAlarm(0);
            return (GWTAlarm[]) values.toArray(new GWTAlarm[0]);
        }

    }

    public void ackAlarm(String id) {
        String system;
        try {
            system = (String) server.getAttribute(target, "ServerId");
            String user = System.getProperty("user.name");
            String alarmId = id;
            if (alarmId.equals("*")){
                server
                        .invoke(target, "acknowledgeAll", new Object[] { user,
                                system }, new String[] { STATIC_STRING,
                                STATIC_STRING });
                alarmSize.source = "0";
            }
            else{
                server.invoke(target, "acknowledge", new Object[] { alarmId,
                        user, system }, new String[] { STATIC_STRING,
                        STATIC_STRING, STATIC_STRING });
                if(altab.length > 0){
                alarmSize = new GWTAlarm(altab.length-1);
                }else{
                    alarmSize.source = "0";
                }
            }

        } catch (AttributeNotFoundException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        } catch (InstanceNotFoundException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        } catch (MBeanException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        } catch (ReflectionException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }

    }

    public void ackAlarm(GWTAlarm[] alarms) {
        String system;

        for (int i = 0; i < alarms.length; i++) {
            try {
                system = (String) server.getAttribute(target, "ServerId");
                String user = System.getProperty("user.name");
                String alarmId = alarms[i].alarmID;
                if (alarmId.equals("*"))
                    server.invoke(target, "acknowledgeAll", new Object[] {
                            user, system }, new String[] { STATIC_STRING,
                            STATIC_STRING });
                else
                    server.invoke(target, "acknowledge", new Object[] {
                            alarmId, user, system }, new String[] {
                            STATIC_STRING, STATIC_STRING, STATIC_STRING });

            } catch (AttributeNotFoundException e) {
                // TODO Auto-generated catch block
                e.printStackTrace();
            } catch (InstanceNotFoundException e) {
                // TODO Auto-generated catch block
                e.printStackTrace();
            } catch (MBeanException e) {
                // TODO Auto-generated catch block
                e.printStackTrace();
            } catch (ReflectionException e) {
                // TODO Auto-generated catch block
                e.printStackTrace();
            }
        }
    }

}
